@@ -83,8 +83,8 @@
 # 
 ##############################################################################
 
-'''$Id: db.py,v 1.29 2001/11/09 01:05:25 fog Exp $'''
-__version__='$Revision: 1.29 $'[11:-2]
+'''$Id: db.py,v 1.29.2.4 2002/03/27 15:40:26 fog Exp $'''
+__version__='$Revision: 1.29.2.4 $'[11:-2]
 
 
 from Shared.DC.ZRDB.TM import TM
@@ -109,40 +109,35 @@ class DB(TM,dbi_db.DB):
 
     _p_oid = _p_changed = _registered = None
 
-    Database_Connection = psycopg.connect
+    connect = psycopg.connect
 
     def __init__(self, connection):
         self.connection = connection
-        try:
-            self.db = self.Database_Connection(self.connection)
-        except psycopg.Error:
-            raise sys.exc_type, sys.exc_value, sys.exc_traceback
-	self.cursor = self.db.cursor()
-        
-    def _begin(self):
-	#we want to begin the transaction when the TM start its work
-        if self.db: self.db.rollback()
-	
+        self.db = self.connect(self.connection)
+    
     def _finish(self, *ignored):
-        if self.db:
-            try:
-                self.db.commit()
-            except psycopg.Error:
-                raise sys.exc_type, sys.exc_value, sys.exc_traceback
+        if self.db: self.db.commit()
             
     def _abort(self, *ignored):
-        if self.db:
-            try:
-                self.db.rollback()
-            except psycopg.Error:
-                pass
+        try:
+            self.db.rollback()
+        except psycopg.Error:
+            pass
 
     def tables(self, rdb=0, _care=('TABLE', 'VIEW')):
         c = self.db.cursor()
         c.execute('SELECT t.tablename AS NAME, '
                   '\'TABLE\' AS TYPE FROM pg_tables t '
+                  'WHERE tableowner <> \'postgres\' '
                   'UNION SELECT v.viewname AS NAME, '
-                  '\'VIEW\' AS TYPE FROM pg_views v ')
+                  '\'VIEW\' AS TYPE FROM pg_views v '
+                  'WHERE viewowner <> \'postgres\' '
+                  'UNION SELECT t.tablename AS NAME, '
+                  '\'SYSTEM_TABLE\' AS TYPE FROM pg_tables t '
+                  'WHERE tableowner = \'postgres\' '
+                  'UNION SELECT v.viewname AS NAME, '
+                  '\'SYSTEM_TABLE\' AS TYPE FROM pg_views v '
+                  'WHERE viewowner = \'postgres\' ' )
         r = []
         a = r.append
         for name, typ in c.fetchall():
@@ -154,7 +149,7 @@ class DB(TM,dbi_db.DB):
     def columns(self, table_name):
         c = self.db.cursor()
         try:
-            r = c.execute('select * from %s limit 1', table_name)
+            r = c.execute('select * from %s where 1=0' % table_name)
         except:
             return ()
         desc = c.description
@@ -188,8 +183,9 @@ class DB(TM,dbi_db.DB):
         desc = ()
         result = []
         self._register()
-        c = self.cursor
+        c = self.db.cursor()
         nselects = 0
+
         try:
             for qs in filter(None, map(strip, split(query_string, '\0'))):
                 try:
@@ -203,39 +199,34 @@ class DB(TM,dbi_db.DB):
                 if c.description is not None:
                     nselects = nselects + 1
                     if c.description != desc and nselects > 1:
-                        raise 'Query Error', (
-                            'Multiple select schema are not allowed')
+                        raise 'Query Error', \
+                              'Multiple select schema are not allowed'
                     if max_rows:
                         result = c.fetchmany(max_rows)
                     desc = c.description
             failures = 0
         except psycopg.OperationalError, err:
             try:
-                del c
-                self.db = self.Database_Connection(self.connection)
-                self.cursor = self.db.cursor()
+                del c ; del self.db
+                self.db = self.connect(self.connection)
             except:
                 raise sys.exc_type, sys.exc_value, sys.exc_traceback
             failures = 0
             return self.query(query_string, max_rows)
         except psycopg.InterfaceError, err:
             if self.db is not None:
-                del c
-                self.db = self.Database_Connection(self.connection)
-                self.cursor = self.db.cursor()
+                del c ; del self.db
+                self.db = self.connect(self.connection)
                 return self.query(query_string, max_rows)
             else:
                 raise sys.exc_type, sys.exc_value, sys.exc_traceback
         except psycopg.Error, err:
             failures = failures + 1
             if (failures > 1000):
-                #try to open a new connection if this one had more
-                #than 1000 errors
                 failures = 0
                 try:
-                    del c
-                    self.db = self.Database_Connection(self.connection)
-                    self.cursor = self.db.cursor()
+                    del c ; del self.db
+                    self.db = self.connect(self.connection)
                 except:
                     raise sys.exc_type, sys.exc_value, sys.exc_traceback
                 return self.query(query_string, max_rows)
@@ -266,8 +257,6 @@ class DB(TM,dbi_db.DB):
 
     def close(self):
         """Close the connection."""
-        self.cursor.close()
         self.db.close()
         self.db = None
-        self.cursor = None
         
