@@ -21,7 +21,7 @@
 
 /*
  * connection.c -- defines the connection object DBAPI 2.0
- * $Id: connection.c,v 1.47.2.1 2002/01/16 14:19:06 fog Exp $
+ * $Id: connection.c,v 1.47.2.2 2002/04/03 08:02:52 fog Exp $
  */
 
 #include <assert.h>
@@ -178,13 +178,72 @@ curs_rollbackall(connobject *self)
 
 static char psyco_conn_close__doc__[] = "Closes the connection.";
 
+static void
+_psyco_conn_close(connobject *self)
+{
+    int len, i;
+    PyObject *tmpobj = NULL;
+    connkeeper *keeper;
+    
+    Dprintf("_psyco_conn_close(): shutting down postgres connections\n");
+
+    curs_closeall(self);
+
+    /* orphans all the children cursors but do NOT destroy them (note that
+       we need to lock the keepers used by the cursors before destroying the
+       connection, else we risk to be inside an execute while we set pgconn
+       to NULL) */
+    assert(self->cursors != NULL);
+    len = PyList_Size(self->cursors);
+    Dprintf("_psyco_conn_close(): len(self->cursors) = %d\n", len);
+    for (i = 0; i < len; i++) {
+        tmpobj = PyList_GetItem(self->cursors, 0);
+        assert(tmpobj);
+        Py_INCREF(tmpobj);
+        PySequence_DelItem(self->cursors, 0);
+        ((cursobject *)tmpobj)->conn = NULL; /* orphaned */
+        Dprintf("_psyco_conn_close(): cursor refcnt = %d\n",
+                tmpobj->ob_refcnt);
+    }
+
+    /* close all the open postgresql connections */
+    len = PyList_Size(self->avail_conn);
+    Dprintf("_psyco_conn_close(): len(self->avail_conn) = %d\n", len);
+    for (i = 0; i < len; i++) {
+        tmpobj = PyList_GetItem(self->avail_conn, 0);
+        assert(tmpobj);
+        Py_INCREF(tmpobj);
+        if ((keeper = (connkeeper *)PyCObject_AsVoidPtr(tmpobj))) {
+            Dprintf("_psyco_conn_close(): destroying avail_conn[%i] at %p\n",
+                    i, keeper);
+            if(keeper) {
+                PQfinish(keeper->pgconn);
+                pthread_mutex_destroy(&(keeper->lock));
+                free(keeper);
+            }
+        }
+        PySequence_DelItem(self->avail_conn, 0);
+        Py_DECREF(tmpobj);
+    }
+    
+    Py_DECREF(self->cursors);
+    Py_DECREF(self->avail_conn);
+
+    /* orphan default cursor and destroy it (closing the last connection to the
+       database) */
+    self->stdmanager->conn = NULL;
+    Py_DECREF(self->stdmanager);
+}
+
 static PyObject *
 psyco_conn_close(connobject *self, PyObject *args)
 {
     EXC_IFCLOSED(self);
     PARSEARGS(args);
-    self->closed = 1;  /* now the connection is considered closed */
-    curs_closeall(self);
+
+    /* from now on the connection is considered closed */
+    self->closed = 1;  
+    _psyco_conn_close(self);
     Dprintf("psyco_conn_close(): connection closed\n");
     Py_INCREF(Py_None);
     return Py_None;
@@ -378,58 +437,7 @@ psyco_conn_setattr(connobject *self, char *name, PyObject *v)
 static void
 psyco_conn_destroy(connobject *self)
 {
-    int len, i;
-    PyObject *tmpobj = NULL;
-    connkeeper *keeper;
-    
-    Dprintf("psyco_conn_destroy(): refcnt = %d\n", self->ob_refcnt);
-
-    curs_closeall(self);
-
-    /* orphans all the children cursors but do NOT destroy them (note that
-       we need to lock the keepers used by the cursors before destroying the
-       connection, else we risk to be inside an execute while we set pgconn
-       to NULL) */
-    assert(self->cursors != NULL);
-    len = PyList_Size(self->cursors);
-    Dprintf("psyco_conn_destroy(): len(self->cursors) = %d\n", len);
-    for (i = 0; i < len; i++) {
-        tmpobj = PyList_GetItem(self->cursors, 0);
-        assert(tmpobj);
-        Py_INCREF(tmpobj);
-        PySequence_DelItem(self->cursors, 0);
-        ((cursobject *)tmpobj)->conn = NULL; /* orphaned */
-        Dprintf("psyco_conn_destroy(): cursor refcnt = %d\n",
-                tmpobj->ob_refcnt);
-    }
-
-    /* close all the open postgresql connections */
-    len = PyList_Size(self->avail_conn);
-    Dprintf("psyco_conn_destroy(): len(self->avail_conn) = %d\n", len);
-    for (i = 0; i < len; i++) {
-        tmpobj = PyList_GetItem(self->avail_conn, 0);
-        assert(tmpobj);
-        Py_INCREF(tmpobj);
-        if ((keeper = (connkeeper *)PyCObject_AsVoidPtr(tmpobj))) {
-            Dprintf("psyco_conn_destroy(): destroying avail_conn[%i] at %p\n",
-                    i, keeper);
-            if(keeper) {
-                PQfinish(keeper->pgconn);
-                pthread_mutex_destroy(&(keeper->lock));
-                free(keeper);
-            }
-        }
-        PySequence_DelItem(self->avail_conn, 0);
-        Py_DECREF(tmpobj);
-    }
-    
-    Py_DECREF(self->cursors);
-    Py_DECREF(self->avail_conn);
-
-    /* orphan default cursor and destroy it (closing the last connection to the
-       database) */
-    self->stdmanager->conn = NULL;
-    Py_DECREF(self->stdmanager);
+    _psyco_conn_close(self);
     pthread_mutex_destroy(&(self->lock));
     free(self->dsn);
     PyObject_Del(self);
