@@ -20,8 +20,8 @@
  */
 
 /*
- * cursor.c -- defines the cursor object DBAPI 2.0
- * $Id: cursor.c,v 1.102.2.2 2002/01/17 18:47:55 fog Exp $
+ * cursor.c - defines the cursor object DBAPI 2.0
+ * $Id: cursor.c,v 1.102.2.5 2002/03/27 15:40:26 fog Exp $
  */
 
 #include <assert.h>
@@ -884,26 +884,56 @@ _psyco_curs_execute(cursobject *self, char *query)
 
 /* _mogrify_dict() _mogrify_sequence() - used to implement string quoting */
 
+static char *
+_mogrify_fmt(char *s, char x)
+{
+    if (!s) return NULL;
+    
+    while (1) {
+        while (*s && *s != '%') s++;
+        if (!*s) return NULL;
+        
+        switch (*(++s)) {
+        case '\0':
+            return NULL;
+
+        case '(':
+            while (*s && *s != ')') s++;
+            if (!*s || !*(++s)) return NULL;
+            if (x) *s = x;
+            return s;
+            
+        case '%':
+            break;
+
+        default:
+            if (x) *s = x;
+            return s;
+        }
+    }
+}
+
 static PyObject *
-_mogrify_dict(PyObject *dict)
+_mogrify_dict(PyObject *dict, PyObject *fmt)
 {
     int pos = 0;
+    char *s = PyString_AsString(fmt);
     PyObject *new_dict, *key, *value;
 
     new_dict = PyDict_New();
 
     while (PyDict_Next(dict, &pos, &key, &value)) {
-        /* Py_INCREF(key); */
         if (PyString_Check(value)) {
             PyDict_SetItem(new_dict, key, new_psyco_quotedstringobject(value));
+            s = _mogrify_fmt(s, 0);
         }
         else if (value == Py_None) {
             PyDict_SetItem(new_dict, key, PyString_FromString("NULL"));
-            Py_DECREF(Py_None);
+            s = _mogrify_fmt(s, 's');
         }
         else {
             PyDict_SetItem(new_dict, key, value);
-            /* Py_INCREF(value); */
+            s = _mogrify_fmt(s, 0);
         }
     }
 
@@ -911,9 +941,10 @@ _mogrify_dict(PyObject *dict)
 }
 
 static PyObject *
-_mogrify_seq(PyObject *seq)
+_mogrify_seq(PyObject *seq, PyObject *fmt)
 {
     int i, len;
+    char *s = PyString_AsString(fmt);
     PyObject *new_seq, *value;
 
     len = PyObject_Length(seq);
@@ -923,21 +954,23 @@ _mogrify_seq(PyObject *seq)
         value = PySequence_GetItem(seq, i);
         if (PyString_Check(value)) {
             PyTuple_SET_ITEM(new_seq, i, new_psyco_quotedstringobject(value));
+            s = _mogrify_fmt(s, 0);
         }
         else if (value == Py_None) {
             PyTuple_SET_ITEM(new_seq, i, PyString_FromString("NULL"));
             Py_DECREF(Py_None);
+            s = _mogrify_fmt(s, 's');
         }
         else {
             PyTuple_SET_ITEM(new_seq, i, value);
             Py_INCREF(value);
+            s = _mogrify_fmt(s, 0);
         }
     }
 
     return new_seq;
 }
 
-
 /* psyco_curs_execute() - prepare and execute a database query */
 
 static char psyco_curs_execute__doc__[] = 
@@ -964,8 +997,8 @@ psyco_curs_execute(cursobject *self, PyObject *args)
        we do it using python methods */
     if (d)
     {
-        if (PyDict_Check(d)) { d = _mogrify_dict(d); }
-        else if (PySequence_Check(d)) { d = _mogrify_seq(d); }
+        if (PyDict_Check(d)) { d = _mogrify_dict(d, operation); }
+        else if (PySequence_Check(d)) { d = _mogrify_seq(d, operation); }
         else {
             PyErr_SetString(PyExc_TypeError,
                         "execute, argument 2: expected sequence or mapping");
