@@ -20,7 +20,7 @@
 
 /*
  * typemod.c -- types available to the psycopg module
- * $Id: typemod.c,v 1.9 2001/10/31 16:33:33 fog Exp $
+ * $Id: typemod.c,v 1.9.2.1 2002/04/05 10:45:40 fog Exp $
  */
 
 #include <unistd.h>
@@ -253,7 +253,7 @@ new_psyco_bufferobject(PyObject *buffer)
                     space, quoted);
         }
         if (original[i]) {
-            if (original[i] >= ' ') {
+            if (original[i] >= ' ' && original[i] <= '~') {
                 if (original[i] == '\'') {
                     *chptr = '\\';
                     chptr++;
@@ -265,23 +265,23 @@ new_psyco_bufferobject(PyObject *buffer)
                     chptr += 4;
                 }
                 else {
-                    /* leave it as it is if >= ' ' */
+                    /* leave it as it is if ascii printable */
                     *chptr = original[i];
                     chptr++;
                 }
             }
             else {
-                char c;
+                unsigned char c;
                 
                 /* escape to octal notation \nnn */
-                *chptr = '\\';
-                chptr++;
+                *chptr++ = '\\';
+                *chptr++ = '\\';
                 c = original[i];
-                *chptr = ((c >> 6) & 0007) | 0060;
+                *chptr = ((c >> 6) & 0x07) + 0x30;
                 chptr++;
-                *chptr = ((c >> 3) & 0007) | 0060;
+                *chptr = ((c >> 3) & 0x07) + 0x30;
                 chptr++;
-                *chptr = (c & 0007) | 0060;
+                *chptr = (c & 0x07) + 0x30;
                 chptr++;
             }
         }
