@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2001,2002 Federico Di Gregorio <fog@debian.org>
  *
  * This file is part of the psycopg module.
  *
@@ -20,7 +20,7 @@
 
 /*
  * typeobj.c -- define the type objects
- * $Id: typeobj.c,v 1.20.2.3 2001/12/29 00:42:32 fog Exp $
+ * $Id: typeobj.c,v 1.20.2.8 2002/03/29 12:53:25 fog Exp $
  */
 
 #include "assert.h"
@@ -66,14 +66,23 @@ psyco_DATE_cast(PyObject *s)
     double ss=0.0;
 
     char *str = PyString_AsString(s);
+
+    /* check for infinity */
+    if (!strcmp(str, "infinity") || !strcmp(str, "-infinity")) {
+        if (str[0] == '-') {
+            return mxDateTimeP->DateTime_FromDateAndTime(-5851455,1,1, 0,0,0);
+        }
+        else {
+            return mxDateTimeP->DateTime_FromDateAndTime(5867440,12,31, 0,0,0);
+        }
+    }
     
     Dprintf("psyco_DATE_cast(): s = %s\n", str);
-    
     n = sscanf(str, "%d-%d-%d %d:%d:%lf", &y, &m, &d, &hh, &mm, &ss);
     Dprintf("psyco_DATE_cast(): date parsed, %d components\n", n);
     
     if (n != 3 && n != 6) {
-        PyErr_SetString(DataError, "unable to parse date");
+        PyErr_SetString(DataError, "unable to parse date or timestamp");
         return NULL;
     }
     return mxDateTimeP->DateTime_FromDateAndTime(y, m, d, hh, mm, ss);
@@ -152,14 +161,14 @@ static PyObject *
 psyco_INTERVAL_cast(PyObject *s)
 {
     long years = 0, months = 0, days = 0;
-    double hours = 0.0, minutes = 0.0, seconds = 0.0, v = 0.0, sign = 1.0;
+    double hours = 0.0, minutes = 0.0, seconds = 0.0, hundredths = 0.0;
+    double v = 0.0, sign = 1.0;
     int part = 0;
     char *str;
 
     str = PyString_AsString(s);
     Dprintf("psyco_INTERVAL_cast(): s = %s\n", str);
-    Dprintf("hey! postgres guys, do you want to make me crazy?\n");
-
+    
     while (*str) {
         switch (*str) {
 
@@ -216,6 +225,14 @@ psyco_INTERVAL_cast(PyObject *s)
             }
             break;
 
+        case '.':
+            if (part == 5) {
+                seconds = v;
+                Dprintf("psyco_INTERVAL_cast(): seconds = %f\n", seconds);
+                v = 0.0; part = 6;
+            }
+            break;   
+
         default:
             break;
         }
@@ -223,7 +240,7 @@ psyco_INTERVAL_cast(PyObject *s)
         str++;
     }
 
-    /* manage last value, be it minutes or seconds */
+    /* manage last value, be it minutes or seconds or hundredths of a second */
     if (part == 4) {
         minutes = v;
         Dprintf("psyco_INTERVAL_cast(): minutes = %f\n", minutes);
@@ -232,13 +249,19 @@ psyco_INTERVAL_cast(PyObject *s)
         seconds = v;
         Dprintf("psyco_INTERVAL_cast(): seconds = %f\n", seconds);
     }
+    else if (part == 6) {
+        hundredths = v;
+        Dprintf("psyco_INTERVAL_cast(): hundredths = %f\n",
+                hundredths);
+    }
+    
 
     /* calculates seconds */
     if (hours < 0.0) {
-        seconds = - seconds - minutes*60 + hours*3600;
+        seconds = - (hundredths/100.0 + seconds + minutes*60) + hours*3600;
     }
     else {
-        seconds += minutes*60 + hours*3600;
+        seconds += hundredths/100.0 + minutes*60 + hours*3600;
     }
 
     /* calculates days */
@@ -367,12 +390,13 @@ psyco_add_type(PyObject *obj)
     psyco_DBAPITypeObject *type = (psyco_DBAPITypeObject *)obj;
     PyObject *val;
     int len, i;
-    
+
+    Dprintf("psyco_add_type: typeobject = %p, values refcnt = %d\n",
+            obj, type->values->ob_refcnt);
     len = PyTuple_Size(type->values);
     for (i = 0; i < len; i++) {
         val = PyTuple_GetItem(type->values, i);
-        Dprintf("psyco_add_type: val = %ld, typeobject = %p\n",
-                PyInt_AsLong(val), obj);
+        Dprintf("psyco_add_type: val = %ld\n",PyInt_AsLong(val));
         PyDict_SetItem(psyco_types, val, obj);
     }
 
@@ -496,8 +520,9 @@ psyco_DBAPITypeObject_destroy(psyco_DBAPITypeObject *self)
     name = self->name;
     cast = self->pcast;
 
-    
-    Dprintf("psyco_DBAPITypeObject_destroy: after value refcnt = %d\n",
+    Dprintf("psyco_DBAPITypeObject_destroy: value refcnt = %d\n",
+            values->ob_refcnt);
+    Dprintf("psyco_DBAPITypeObject_destroy: value[0] refcnt = %d\n",
             PyTuple_GET_ITEM(self->values, 0)->ob_refcnt);
     PyObject_Del(self);
 
@@ -583,9 +608,6 @@ static PyObject *
 psyco_DBAPITypeObject_new(PyObject *name, PyObject *values, PyObject *cast)
 {
     psyco_DBAPITypeObject *obj;
-
-    Dprintf("psyco_DBAPITypeObject_new: name = %p, values = %p, cast = %p\n",
-            name, values, cast);
     
     obj = PyObject_NEW(psyco_DBAPITypeObject, &psyco_DBAPITypeObject_Type);
     if (obj == NULL) return NULL;
